@@ -2,8 +2,10 @@
 #define INCLUDED_APPEND_LIST_H
 
 #include "struct_VList.h"
+#include "xmalloc.h" // inline
 
-#include "xmalloc.h"
+static VList *now;
+static VList *before_last;
 
 // 関数プロトタイプ
 static inline VList * append_list(VList *first, const void *tmp) __attribute__((always_inline));
@@ -15,8 +17,8 @@ static inline VList * append_list(VList *first, const void *tmp)
 {
 	if(first != NULL)
 	{
-		VList *now = xmalloc(sizeof(VList));
-		VList *before_last = first->tail;
+		now = xmalloc(sizeof(VList));
+		before_last = first->tail;
 		/*
 		* キャストしないと警告が出る
 		* warning: initialization discards qualifiers from pointer target type
@@ -30,12 +32,13 @@ static inline VList * append_list(VList *first, const void *tmp)
 
 		first->tail = now;
 	}
-	// リストに最初の要素を追加する
 	else
 	{
+		// リストに最初の要素を追加する
+
 		first = xmalloc(sizeof(VList));
 		/*
-		* キャストしないと警告が出る…
+		* キャストしないと警告が出る
 		* warning: initialization discards qualifiers from pointer target type
 		*/
 		first->data = (void *)tmp;
