@@ -1,8 +1,7 @@
 #define _GNU_SOURCE
 
 #include "struct_SDir.h"
-
-#include "xmalloc.h"
+#include "xmalloc.h" // inline
 
 #include <errno.h>
 #include <fcntl.h>
@@ -18,6 +17,7 @@ SDir * get_settings_dir(void)
 {
 	// g_getenvの戻り値は const gchar *
 	const char *homedir = g_getenv("HOME");
+
 	if(!homedir)
 	{
 		homedir = g_get_home_dir();
@@ -32,10 +32,6 @@ SDir * get_settings_dir(void)
 		sdir->user_settings_dir = tmp; /* ~/.config/snowcp */
 	}
 
-	/*
-	 * ログフォルダとカレントを行き来するためのファイルディスクリプタ。
-	 * だったんだけど、使わなくなった…。
-	*/
 	errno = 0;
 	if((sdir->current_fd = open(".", O_RDONLY | O_NOATIME)) == -1)
 	{
