@@ -7,11 +7,11 @@
 
 #include <errno.h>
 
-static inline void print_error(char *func, char *file, int line, CPDD *cpdd) __attribute__((always_inline));
+static inline void print_error(const char *func, const char *file, const int line, CPDD *cpdd) __attribute__((always_inline));
 
 /*******************************************************************************
 *******************************************************************************/
-static inline void print_error(char *func, char *file, int line, CPDD *cpdd)
+static inline void print_error(const char *func, const char *file, const int line, CPDD *cpdd)
 {
 	printf("%s : %d行\n", file, line);
 	perror(func);
