@@ -18,6 +18,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <linux/falloc.h>
 #include <pthread.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -226,6 +227,8 @@ _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 	 * ext4からfallocateシステムコールがサポートされた。
 	 * sparse file化とは違うらしい？
 	 * ext4以外の、それをサポートしていないファイルシステムだとかなり遅くなる様子。
+	 * ext4だと4秒弱で終わるフォルダのコピーが、ntfsだと12秒はかかる。
+	 * （何もしないと6秒弱、pwriteだと8秒ほど、ftruncateは何もしない場合とほぼ同じ）
 	*/
 	if((from_size_local > 0) && (to_open == true))
 	{
@@ -239,25 +242,29 @@ _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 		(tofs_tmp[3] == '4')
 		)
 		{
-			errno = 0;
-			if(posix_fallocate(ito, 0, from_size_local) != 0)
-			{
-				print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
+			static int itmp = -1;
 
-				errno = 0;
-				if(ftruncate(ito, from_size_local) == -1)
-				{
-					print_error("ftruncate", __FILE__, __LINE__, cpdd);
-				}
+			// Ubuntu 11.04だとfallocateの第二引数は FALLOC_FL_KEEP_SIZE と FALLOC_FL_PUNCH_HOLE の二つ。
+			// linux/falloc.h のincludeが必要。
+			if((itmp = fallocate(ito, FALLOC_FL_KEEP_SIZE, 0, from_size_local)) != 0)
+			{
+				errno = itmp;
+				print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
 			}
 		}
 		else
 		{
 			errno = 0;
+			if(ftruncate(ito, from_size_local) == -1)
+			{
+				print_error("ftruncate", __FILE__, __LINE__, cpdd);
+			}
+/*
 			if(pwrite(ito, "", 1, from_size_local - 1) == -1)
 			{
 				print_error("pwrite", __FILE__, __LINE__, cpdd);
 			}
+*/
 		}
 	}
 
@@ -429,12 +436,26 @@ LOOP_END:
 		 * ここを弄ってバージョン0.1.2でバグを出したので、うかつに弄らない。
 		 * 
 		 * 0.4.0から、
-		 * openしたときにposix_fallocate、ftruncate、pwriteのいずれかを、
+		 * openした後にposix_fallocate、fallocate、ftruncate、pwriteのいずれかを、
 		 * 必ず行うようにしたが、念のためここの処理も残しておくべき。
 		*/
-		if(ftruncate(ito, from_size_local) == -1)
 		{
-			print_error("ftruncate", __FILE__, __LINE__, cpdd);
+			static struct stat to_stat_tmp;
+
+			if(fstat(ito, &to_stat_tmp) == -1)
+			{
+				print_error("fstat", __FILE__, __LINE__, cpdd);
+			}
+			else if(from_size_local >= to_stat_tmp.st_size)
+			{
+			}
+			else if(ftruncate(ito, from_size_local) != -1)
+			{
+			}
+			else
+			{
+				print_error("ftruncate", __FILE__, __LINE__, cpdd);
+			}
 		}
 	}
 
@@ -535,6 +556,7 @@ LOOP_END:
 static void * read_thread(void *unknown)
 {
 	int i = rbuf_num;
+	unknown++;	// コンパイル時のwarningを出さないようにするため
 
 	if(from_size_local <= total_read_local)
 	{
@@ -562,14 +584,12 @@ static void * read_thread(void *unknown)
 	if(buf[i] != MAP_FAILED)
 	{
 		errno = 0;
-		/*
-		 * マルチスレッドの場合、
-		 * MADV_SEQUENTIALよりもMADV_WILLNEEDの方が速い、ような気がする。
-		*/
+		// マルチスレッドの場合、
+		// MADV_SEQUENTIALよりもMADV_WILLNEEDの方が速い、ような気がする。
 		if(madvise(buf[i], read_size, MADV_WILLNEED) != 0)
 		{
 			// このcpddに意味はない。print_errorの引数用。
-			CPDD cpdd;
+			static CPDD cpdd;
 			cpdd.total_error = 0;
 			print_error("madvise", __FILE__, __LINE__, &cpdd);
 			total_read_error++;
@@ -594,6 +614,7 @@ static void * write_thread(void *unknown)
 	int i = wbuf_num;
 	char *tmp = (char *)buf[i];
 	const char * const endp = tmp + next_write_size[i];
+	unknown++;	// コンパイル時のwarningを出さないようにするため
 
 	if(next_write_size[i] > 0)
 	{
@@ -603,7 +624,7 @@ static void * write_thread(void *unknown)
 			if((written_size[i] = write(ito, tmp, endp - tmp)) == -1)
 			{
 				// このcpddに意味はない。print_errorの引数用。
-				CPDD cpdd;
+				static CPDD cpdd;
 				cpdd.total_error = 0;
 				print_error("write", __FILE__, __LINE__, &cpdd);
 				total_write_error++;
@@ -616,7 +637,8 @@ static void * write_thread(void *unknown)
 		errno = 0;
 		if(munmap(buf[i], next_write_size[i]) == -1)
 		{
-			CPDD cpdd;
+			// このcpddに意味はない。print_errorの引数用。
+			static CPDD cpdd;
 			cpdd.total_error = 0;
 			print_error("munmap", __FILE__, __LINE__, &cpdd);
 			total_write_error++;
